@@ -1,10 +1,14 @@
+from typing import List, Union
+
 from pyspark.sql import DataFrame
 
 from atc.atc_exceptions import AtcException
 from atc.config_master import TableConfigurator
-from atc.functions import init_dbutils
+from atc.functions import get_unique_tempview_name, init_dbutils
 from atc.spark import Spark
 from atc.tables.TableHandle import TableHandle
+from atc.utils.CheckDfMerge import CheckDfMerge
+from atc.utils.GetMergeStatement import GetMergeStatement
 
 
 class DeltaHandleException(AtcException):
@@ -31,22 +35,29 @@ class DeltaHandle(TableHandle):
     def from_tc(cls, id: str) -> "DeltaHandle":
         tc = TableConfigurator()
         return cls(
-            name=tc.table_name(id),
+            name=tc.table_property(id, "name", ""),
             location=tc.table_property(id, "path", ""),
             data_format=tc.table_property(id, "format", "delta"),
         )
 
     def _validate(self):
         """Validates that the name is either db.table or just table."""
-        name_parts = self._name.split(".")
-        if len(name_parts) == 1:
-            self._db = None
-            self._table_name = name_parts[0]
-        elif len(name_parts) == 2:
-            self._db = name_parts[0]
-            self._table_name = name_parts[1]
+        if not self._name:
+            if not self._location:
+                raise DeltaHandleInvalidName(
+                    "Cannot create DeltaHandle without name or path"
+                )
+            self._name = f"delta.`{self._location}`"
         else:
-            raise DeltaHandleInvalidName(f"Could not parse name {self._name}")
+            name_parts = self._name.split(".")
+            if len(name_parts) == 1:
+                self._db = None
+                self._table_name = name_parts[0]
+            elif len(name_parts) == 2:
+                self._db = name_parts[0]
+                self._table_name = name_parts[1]
+            else:
+                raise DeltaHandleInvalidName(f"Could not parse name {self._name}")
 
         # only format delta is supported.
         if self._data_format != "delta":
@@ -99,6 +110,64 @@ class DeltaHandle(TableHandle):
         self.drop()
         self.create_hive_table()
 
-    def merge(self):
-        # This should be a method which can merge from delta to delta
-        raise NotImplementedError
+    def get_tablename(self) -> str:
+        return self._name
+
+    def upsert(
+        self,
+        df: DataFrame,
+        join_cols: List[str],
+    ) -> Union[DataFrame, None]:
+
+        if df is None:
+            return None
+
+        df = df.filter(" AND ".join(f"({col} is NOT NULL)" for col in join_cols))
+        print(
+            "Rows with NULL join keys found in input dataframe"
+            " will be discarded before load."
+        )
+
+        df_target = self.read()
+
+        # If the target is empty, always do faster full load
+        if len(df_target.take(1)) == 0:
+            return self.write_or_append(df, mode="overwrite")
+
+        # Find records that need to be updated in the target (happens seldom)
+
+        # Define the column to be used for checking for new rows
+        # Checking the null-ness of one right row is sufficient to mark the row as new,
+        # since null keys are disallowed.
+
+        df, merge_required = CheckDfMerge(
+            df=df,
+            df_target=df_target,
+            join_cols=join_cols,
+            avoid_cols=[],
+        )
+
+        if not merge_required:
+            return self.write_or_append(df, mode="append")
+
+        temp_view_name = get_unique_tempview_name()
+        df.createOrReplaceTempView(temp_view_name)
+
+        target_table_name = self.get_tablename()
+        non_join_cols = [col for col in df.columns if col not in join_cols]
+
+        merge_sql_statement = GetMergeStatement(
+            merge_statement_type="delta",
+            target_table_name=target_table_name,
+            source_table_name=temp_view_name,
+            join_cols=join_cols,
+            insert_cols=df.columns,
+            update_cols=non_join_cols,
+            special_update_set="",
+        )
+
+        Spark.get().sql(merge_sql_statement)
+
+        print("Incremental Base - incremental load with merge")
+
+        return df
